/*	
	Client application (distributed failure detection)
	for benchmarking

	Pedro Martins <pmartins@di.fc.ul.pt>	

--[TTCB 1.1]----------------------------------------------------

    (Trusted) Timely Computing Base 1.1
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/system_defs.h"

#ifdef DIST_FAULT

#include "tcb/tcb.h"
#include <signal.h>
#include <sched.h>
#include <netinet/in.h>
#include <linux/inet.h>

char child_stack[1000];
char child_stack_end;

int thrd1;
int sock_fd;

char p[4324];

descriptor_t tcbDesc;
char ip_addr[3000];

#define CLIENT_TCB_PORT		80
#define SERVER_TCB_PORT		81

void exit_on_error(char * msg)
{
	if (errno==0)
	 fprintf(stderr,"%s",msg);
	  else
	   perror(msg);
	
	printf("\n");
   
	TCBstopwinfoport(CLIENT_TCB_PORT,&tcbDesc); 	    
	kill(thrd1,SIGINT);   
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0);   
}

void sigint_handler(int par)
{
	TCBstopwinfoport(CLIENT_TCB_PORT,&tcbDesc); 	 	
	kill(thrd1,SIGINT);    	
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0);
}

int fn(void* arg)
{
	fail_info_to_app fi;
	
	while (1)
	{
	  TCBwaitInfo(tcbDesc.wi_orig,&fi);  	
	}
	return 0;
}
#endif

int main(int argc,char* argv[])
{
#ifdef DIST_FAULT
	int res;	
	int ret;	
	TCBtimestamp sndtime,rcv_evt;
	unsigned int tag;
	TCBtimeinterval delay,error;	
	Func_Id p;
	int one;
 	struct sockaddr_in cnt,serv;
 	int len; 	
 	char buffer[sizeof(distAction_id_t)];
 	distAction_id_t * distAction;
	int t,a;	
	unsigned char mask=0x1F;
        unsigned char andmask=1; 
	        	
	if ((ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}
	
	if ((t=getTCBid(&tcbDesc))<OK)
	{
		printf("Error on geting the ID of the TCB\n");
		closeTCBConnection(&tcbDesc);   
		exit(1);
	}
	
	if (NUMBER_OF_MACHINES!=1)
	{
	
	for (a=0;a<t;a++)
	 andmask<<=1;		
		
	 andmask=~andmask;
	 mask=mask & andmask;		
	}
	
	//create a thread to read WaitInfo data  	
//	thrd1=clone(&fn,&child_stack_end,CLONE_VM,(void*)0);

	signal(SIGINT,sigint_handler);
	
	if ((sock_fd=socket(AF_INET,SOCK_DGRAM,0))<0)
	{
		exit_on_error("Error on creating an UDP socket...\n\n");
	}

	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR|SO_BROADCAST,&one,sizeof(one));

 	bzero((char*)&cnt,0,sizeof(cnt));
	
 	cnt.sin_family=AF_INET;
 	cnt.sin_addr.s_addr=htonl(INADDR_ANY);
 	cnt.sin_port=htons(0);
 	
 	if ((ret=bind(sock_fd,(struct sockaddr*)&cnt,sizeof(cnt)))!=0)
 	 {	
 	 	exit_on_error("Error on binding an UDP socket...\n");
 	 }

 	bzero((char*)&serv,0,sizeof(serv));
 	
 	serv.sin_family=AF_INET;
 	serv.sin_addr.s_addr=htonl(IP_BROADCAST_ADDR);
 	serv.sin_port=htons(16006);
 	
 	strcpy(p.module,"app_module"); 
 	strcpy(p.funcname,"timelyExecDFD"); 
 	strcpy(p.wcetname,"timelyExec_wcet");  
 	
 	while (1)
 	{
 	 ret = getTCBGlobalTimestamp(&tcbDesc, &sndtime); 	 
 	 if (ret < 0){
		exit_on_error("On getting a global timestamp from the TTCB...\n");
	 }	
 	 
 	 distAction=(distAction_id_t*)&buffer;
 	 ret = startDistFD(&tcbDesc,mask,SERVER_TCB_PORT,sndtime,1000000,0,p,distAction);
 
 	 if (ret<0)
  	  {	
  	  	if ((ret!=ALREADY_FAILED)&&(ret!=BURSTY_SENDS)&&(ret!=TOO_MANY_DISTRIBUTED_FD)){
  			printf("%d\n",ret);
  			exit_on_error("On starting a distributed FD operation...\n");
			exit(1);
  		} 
  		usleep(1);	 	  		
  	  }
  	 else
  		if ((ret=sendto(sock_fd,(char*)buffer,sizeof(distAction_id_t),0,(struct sockaddr*)&serv,sizeof(serv)))==-1)
  	  	{
  	  	  exit_on_error("On sending a message to an UDP socket...\n");
  	  	}
  	
 	}
#else 
	while (1){};
#endif 
}

