/* rsa.h
 *	Copyright (C) 1997,1998 by Werner Koch (dd9jn)
 *	Copyright (C) 2000, 2001 Free Software Foundation, Inc.
 *
 * This file is part of GnuPG.
 *
 * GnuPG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GnuPG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifndef G10_RSA_H
#define G10_RSA_H

typedef struct {
    MPI n;          /* modulus */
    MPI e;          /* exponent */
} RSA_public_key;


typedef struct {
    MPI n;          /* public modulus */
    MPI e;          /* public exponent */
    MPI d;          /* exponent */
    MPI p;          /* prime  p. */
    MPI q;          /* prime  q. */
    MPI u;          /* inverse of p mod q. */
} RSA_secret_key;

#define PUBKEY_ALGO_RSA        1
#define PUBKEY_ALGO_RSA_E      2     /* RSA encrypt only */
#define PUBKEY_ALGO_RSA_S      3     /* RSA sign only */
#define PUBKEY_USAGE_ENC     2	    /* key is good for encryption */

#define PUBKEY_USAGE_SIG     1	    /* key is good for signatures */

#define is_RSA(a)     ((a)==PUBKEY_ALGO_RSA || (a)==PUBKEY_ALGO_RSA_E \
		       || (a)==PUBKEY_ALGO_RSA_S )
/*-- primegen.c --*/
MPI generate_secret_prime( unsigned nbits );

int test_keys( RSA_secret_key *sk, unsigned nbits );
void generate( RSA_secret_key *sk, int nbits );
int  check_secret_key( RSA_secret_key *sk );
void public(MPI output, MPI input, RSA_public_key *skey );
void secret(MPI output, MPI input, RSA_secret_key *skey );
void log_fatal(char * msg);


int rsa_generate( int algo, unsigned nbits, MPI *skey, MPI **retfactors );
int rsa_check_secret_key( int algo, MPI *skey );
int rsa_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey );
int rsa_decrypt( int algo, MPI *result, MPI *data, MPI *skey );
int rsa_sign( int algo, MPI *resarr, MPI data, MPI *skey );
int rsa_verify( int algo, MPI hash, MPI *data, MPI *pkey,
		    int (*cmp)(void *, MPI), void *opaquev );
unsigned rsa_get_nbits( int algo, MPI *pkey );
const char *rsa_get_info( int algo, int *npkey, int *nskey,
				    int *nenc, int *nsig, int *use );


#endif /*G10_RSA_H*/
