/*
	(TCB)
	Distributed Failure Detection Demo Application

	Pedro Martins <pmartins@di.fc.ul.pt>

--[TTCB 1.1]----------------------------------------------------

    (Trusted) Timely Computing Base 1.1
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/tcb.h"
#include <signal.h>
#include "defs.h"
#include <sched.h>
#include <netinet/in.h>
#include <linux/inet.h>

char child_stack[1000];
char child_stack_end;

int thrd1;
int sock_fd;

char p[4324];

descriptor_t tcbDesc;
char ip_addr[3000];

void exit_on_error(char * msg,int code)
{
	printf("%s ",msg);

	if (errno==0)
	 printTCBerror(code);
	  else
	   perror(msg);
	
	printf("\n");
	
	TCBstopwinfoport(CLIENT_TCB_PORT,&tcbDesc); 	    
	kill(thrd1,SIGINT);   
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0);   
}

void sigint_handler(int par)
{
	TCBstopwinfoport(CLIENT_TCB_PORT,&tcbDesc); 	 
	kill(thrd1,SIGINT);    	
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0);
}

void display_fail_info(fail_info_to_app* fi)
{
	int c;
	int mult=1;
	
	printf("********************************************\n");
	printf("Client:\n");
	printf("Message from: %u, Seq: %u Port: %u\n",fi->from,fi->seq,fi->port);
	printf("********************************************\n");
	
	for (c=0;c<NUMBER_OF_MACHINES;c++)
	 {
	   if (fi->failure_info[c].fail!=2)
	   {
	     if (fi->failure_info[c].fail==1)
	      {
	        printf("Timing failure on the reception (machine %u)\n",c); 
	        mult=1;
	      }
	       else
	        {
	        	printf("Timely reception on machine %u\n",c); 
	        	mult=-1;
	        }
	        
	    if (fi->failure_info[c].lateness_avail==1)
	      {
	        printf("lateness degree=%d\n",fi->failure_info[c].lateness_degree*mult); 
	      }
	       else
	        {
	        	printf("Lateness degree not available\n",c); 
	        }    	
	     printf("\n");            
	   }	   
	 }
	 
}

int fn(void* arg)
{
	int ret;	

	fail_info_to_app fi;
	
	while (1)
	{
	  ret = TCBwaitInfo(tcbDesc.wi_orig,&fi);  	
	  if (ret!=0){
		printf("Error on getting wait info\n");
	}
   	  display_fail_info(&fi);
	}
	return 0;
}


int main(int argc,char* argv[])
{
	int res;	
	int ret;		
	int one=1;
	TCBtimestamp sndtime,rcv_evt, grcv_evt;
	unsigned int tag;
	TCBtimeinterval delay,error;	
	Func_Id p;
 	struct sockaddr_in cnt,serv;
 	int len; 	
 	char buffer[1000];
	void **data;
	unsigned long long cnts;
	pingResp* pt;
	distAction_id_t * distAction;
	fd_set fds;
	struct timeval tv;
	

	printf("\n----------------------------------------------------------\n");
	printf("Distributed failure detection service - client aplication\n");
	printf("----------------------------------------------------------\n\n");
	
	
	if ((ret = openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}
        	
	//create a thread to read the WaitInfo data  	
	thrd1=clone(&fn,&child_stack_end,CLONE_VM,(void*)0);
	
	signal(SIGINT,sigint_handler);
	
	if ((sock_fd=socket(AF_INET,SOCK_DGRAM,0))<0)
	{
		exit_on_error("Error on creating an UDP socket...\n\n",sock_fd);
	}

	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR|SO_BROADCAST,&one,sizeof(one));
	
 	bzero((char*)&cnt,0,sizeof(cnt));
	
 	cnt.sin_family=AF_INET;
 	cnt.sin_addr.s_addr=htonl(INADDR_ANY);
 	cnt.sin_port=htons(0);
 	
 	if ((ret=bind(sock_fd,(struct sockaddr*)&cnt,sizeof(cnt)))!=0)
 	 {	
 	 	exit_on_error("Error on binding an UDP socket...\n",ret);
 	 }
 	
 	bzero((char*)&serv,0,sizeof(serv));
 	
 	serv.sin_family=AF_INET;
 	serv.sin_addr.s_addr=htonl(IP_BROADCAST_ADDR);
 	serv.sin_port=htons(16006);
 	
 	cnts=0;
 	
 	//handler for timely timing failure detection
 	strcpy(p.module,"app_module"); 
 	strcpy(p.funcname,"client_timelyExec"); 
 	strcpy(p.wcetname,"timelyExec_wcet"); 
 	
	ret = TCBwinfoport(CLIENT_TCB_PORT,&tcbDesc); 	 
        if (ret < 0){
		exit_on_error("On setting a winfo reader for a TCB port ...\n",ret);
	}	
 	
 	while (1)
 	{
 	 cnts++;
 	 if ((cnts>400000))  
  	 {
  	 ret = getTCBGlobalTimestamp(&tcbDesc, &sndtime); 	 
 	 if (ret < 0){
		exit_on_error("On getting a global timestamp from the TTCB...\n",ret);
	 }

	distAction=(distAction_id_t*)&buffer;
 	ret = startDistFD(&tcbDesc,DESTINATION_MASK,SERVER_TCB_PORT,sndtime,SPEC,0,p,distAction);
 
 	if (ret < 0){
		exit_on_error("On starting a distributed FD operation...\n",ret);
		exit(1);
	 }

  	 if ((ret=sendto(sock_fd,(char*)buffer,sizeof(distAction_id_t),0,(struct sockaddr*)&serv,sizeof(serv)))==-1)
  	  {
  	  	exit_on_error("On sending a message to an UDP socket...\n",ret);
  	  }
  	cnts=0;
  	}
  
	//Non Blocking reading
  	FD_ZERO(&fds);
	FD_SET(sock_fd,&fds);	 		 	
	 	
	tv.tv_sec=0;
	tv.tv_usec=0;
	 	
	ret=select(sock_fd+1,&fds,NULL,NULL,&tv);
	 	
	if (ret)
	{
 	len=sizeof(cnt);
 	if ((ret=recvfrom(sock_fd,(char*)buffer,sizeof(pingResp)+sizeof(distAction_id_t),0,(struct sockaddr*)&cnt,&len))!=-1) 	
 	{
 	  distAction = (distAction_id_t*)&buffer; 		
 	  if (TFD_ENABLED(distAction))
	 	  ret = endDistFD(&tcbDesc,distAction,&rcv_evt,&grcv_evt,&delay);
	  else {
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("TFD operation expected...\n");
		exit(1);	
	  }

	  if (ret < 0){
		exit_on_error("On ending a distributed FD operation...\n",ret);
	  }
 
 	 pt=(pingResp*)(buffer+sizeof(distAction_id_t));	
 	 printf("Packet From %s:\n",inet_ntoa(cnt.sin_addr.s_addr));
 	 printf("------------------------------------\n");
 	 printf("Received: delay <%llu> us\n",delay/1000);
 	 printf("\n");
 	}	  
 	 else
 	 if (ret<0)  
 	  {
 	     exit_on_error("On getting a message from an UDP socket...\n",ret);
 	  }
 	}  //  !retval  
 	}
}

