/*
	(TCB)
	Distributed Failure Detection Demo Application

	Pedro Martins <pmartins@di.fc.ul.pt>

--[TTCB 1.1]----------------------------------------------------

    (Trusted) Timely Computing Base 1.1
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/tcb.h"
#include <signal.h>
#include "defs.h"
#include <sched.h>
#include <netinet/in.h>
#include <linux/inet.h>

char child_stack[1000];
char child_stack_end;

char p[4324];

int thrd1;

int sock_fd;

descriptor_t tcbDesc;

void exit_on_error(char * msg,int code)
{
	printf("%s ",msg);

	if (errno==0)
	 printTCBerror(code);
	  else
	   perror(msg);
	
	printf("\n");

	TCBstopwinfoport(SERVER_TCB_PORT,&tcbDesc); 	    
	kill(thrd1,SIGINT);      
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(1);   
}

void sigint_handler(int par)
{
	TCBstopwinfoport(SERVER_TCB_PORT,&tcbDesc); 	    
	kill(thrd1,SIGINT);   
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0);
}


void display_fail_info(fail_info_to_app* fi)
{
	int c;
	int mult=1;
	
	printf("********************************************\n");
	printf("Server:\n");
	printf("Message from: %u, Seq: %u Port: %u\n",fi->from,fi->seq,fi->port);
	printf("********************************************\n");
	
	for (c=0;c<NUMBER_OF_MACHINES;c++)
	 {
	   if (fi->failure_info[c].fail!=2)
	   {
	     if (fi->failure_info[c].fail==1)
	      {
	        printf("Timing failure on the reception (machine %u)\n",c); 
	        mult=1;
	      }
	       else
	        {
	        	printf("Timely reception on machine %u\n",c); 
	        	mult=-1;
	        }
	        
	    if (fi->failure_info[c].lateness_avail==1)
	      {
	        printf("lateness degree=%d\n",fi->failure_info[c].lateness_degree*mult); 
	      }
	       else
	        {
	        	printf("Lateness degree not available\n",c); 
	        }    	
	     printf("\n");            
	   }	   
	 }
	 
}

int fn(void* arg)
{
fail_info_to_app fi;
	int ret;	

	while (1)
	{
	ret = TCBwaitInfo(tcbDesc.wi_orig,&fi);  	
	if (ret!=0){
		printf("Error on getting wait info\n");
	}
  	display_fail_info(&fi);
	}
	return 0;
}

int main()
{
 	int res;	
	int ret;
	char ip_addr[300];	
	int one=1;
	TCBtimestamp sndtime,rcv_evt, grcv_evt;
	unsigned int tag;
	TCBtimeinterval delay,error;	
 	struct sockaddr_in serv,cnt;
 	int len; 	
 	char buffer[1000];
	void **data;
	pingResp* pt;
	distAction_id_t * distAction;
	fd_set fds;
	struct timeval tv;
	Func_Id p;
	unsigned char dest_mask=0x01;
	int a;

	
	printf("\n---------------------------------------------------------\n");
	printf("Distributed failure detection service - server aplication\n");
	printf("----------------------------------------------------------\n\n");

	
	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}

	thrd1=clone(&fn,&child_stack_end,CLONE_VM,(void*)0);
	
	signal(SIGINT,sigint_handler);


	if ((sock_fd=socket(AF_INET,SOCK_DGRAM,0))<0)
	{
		exit_on_error("Error on creating an UDP socket...\n\n",sock_fd);
	}

	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR|SO_BROADCAST,&one,sizeof(one));
		
 	bzero((char*)&serv,0,sizeof(cnt));
 	
 	serv.sin_family=AF_INET;
 	serv.sin_addr.s_addr=htonl(INADDR_ANY);
 	serv.sin_port=htons(16006);
 	
	if ((ret=bind(sock_fd,(struct sockaddr*)&serv,sizeof(serv)))!=0)
 	 {	
 	 	exit_on_error("Error on binding an UDP socket...\n",ret);
 	 }
 	
 	
 	//we want to get winfo about the traffic on this port 
 	ret = TCBwinfoport(SERVER_TCB_PORT,&tcbDesc); 	 
        if (ret < 0){
		exit_on_error("On setting a winfo reader for a TCB port ...\n",ret);
	}	
 
 	// handler for timely timing failure detection
 	strcpy(p.module,"app_module"); 
 	strcpy(p.funcname,"server_timelyExec"); 
 	strcpy(p.wcetname,"timelyExec_wcet"); 
 		
 
	while (1)
	{ 	
	len=sizeof(serv); 	

	if ((ret=recvfrom(sock_fd,(char*)buffer,sizeof(distAction_id_t),0,(struct sockaddr*)&cnt,&len))!=-1) 	 	 	 
 	{
 	  distAction = (distAction_id_t*)&buffer; 		
 	  if (TFD_ENABLED(distAction))
	 	  ret = endDistFD(&tcbDesc,distAction,&rcv_evt,&grcv_evt,&delay);
	  else {
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("TFD operation expected...\n");
		exit(1);	
	  }


	  if (ret < 0){
		exit_on_error("On ending a distributed FD operation...\n",ret);
	  }	

	  printf("Packet From %s:\n",inet_ntoa(cnt.sin_addr.s_addr));
 	  printf("------------------------------------\n");
 	  printf("Received: delay <%llu> us\n",delay/1000);
 	  printf("\n");

 	  pt=(pingResp*)(buffer+sizeof(distAction_id_t));	 
 	  pt->delay=delay;	
 
 	  ret = getTCBGlobalTimestamp(&tcbDesc, &sndtime); 	  	
 
 	  if (ret < 0){
		exit_on_error("On getting a global timestamp from the TCB...\n",ret);
	  }

	  //build destination mask to reply to the client (distAction->srcId)	
	  for (a=0;a<distAction->srcId;a++)
		dest_mask = dest_mask <<1;
	  		

	  distAction=(distAction_id_t*)&buffer;
 	  ret = startDistFD(&tcbDesc,dest_mask,CLIENT_TCB_PORT,sndtime,SPEC,0,p,distAction);
 
 	  if (ret < 0){
		exit_on_error("On starting a distributed FD operation...\n",ret);
		exit(1);
  	  }
 
 
	  if ((ret=sendto(sock_fd,(char*)buffer,sizeof(distAction_id_t)+sizeof(pingResp),0,(struct sockaddr*)&cnt,len))==-1)  	
  	   {
  	   	exit_on_error("On sending a message in a TCB socket...\n",ret);
  	   }
 	}
 	else
 	{
 	  exit_on_error("On getting a message from a TCB socket...\n",ret);
 	}	
 	 	
	} 	
}

