/* Agreement test */

#include "tcb/tcb.h"
#include "tcb/ttcb.h"
#include <sys/time.h>

#include <unistd.h>

descriptor_t descriptor;

/*DEFINE THE AGREEMENT TYPE HERE.
  TTCB_TBA_RMULTICAST | TTCB_TBA_MAJORITY | TTCB_TBA_AND | TTCB_TBA_OR | TTCB_TBA_XOR            
*/

#define AGREEMENT_TYPE TTCB_TBA_XOR

void printMask(mask_t mask, int len)  // prints a mask returned by decide
{
	int i;
	mask_t bit = 1;

	for (i = 0; i < len; i++) {
		if ((bit & mask) == 0) printf("0"); else printf("1");
		bit = bit << 1;
	} 
}

void printResult(value_t value)  // prints a mask returned by decide
{
	int i, j;
	unsigned char bit;
	unsigned char * s = (unsigned char *) value;

	for (j = 0; j < sizeof(value_t); j++)
	  {
	    bit = 0x80;
	    
	    for (i = 0; i < 8 ; i++) {
	      if ( (bit & s[j]) == 0x0) printf("0"); else printf("1");
	      bit = bit >> 1 ;
	    } 
	    printf(" ");
	  }
	
}


int main()
{
	propose_out_t outp;
	decide_out_t outd;
	timestamp_out_t ts_out;
	error_out_t err_out;
	unsigned i;
	int curr_time;
	int size_elist;
	elist_t elist;
	value_t value;
	TCBtimestamp tstart;
	TCBtimestamp globalTimestamp;
	int TTCBerror;

// If descriptor is declared global, this is not necessary!
	memset(&descriptor,0, sizeof(descriptor_t));

	if ( (TTCBerror = openTCBConnection(&descriptor))<OK) {
		printTCBerror(TTCBerror);
		exit(1);
		return 1;
	}
	printf("My eid is: %u  (ltid=%u ind=%u)\n", descriptor.eid, descriptor.eid /256, descriptor.eid %256);

	// propose value
	printf("Please introduce the items needed for agreement. Remember that all entities have to introduce the same elist (with the elements in the same order) and the same tstart, or else they will engage in different agreements.\n\n");

	printf("Please introduce the elist (one eid at a time and 0 to terminate): \n");
	i = 0;
	do {
	  scanf("%u", &(elist[i]));
	  i++;
	} while(elist[i-1] > 0);
	elist[i-1] = END_ELIST;
	size_elist = i-1;
	
	printf("Introduce the value to propose: ");
	bzero(&value, sizeof(value));
	scanf("%s", (char *)value);

	TTCBerror = getTCBGlobalTimestamp(&descriptor, &globalTimestamp); //call TTCB
	if (TTCBerror < 0) {
		print_error_TTCB(TTCBerror);
		goto finalize;
	}
	printf("Current time is %lld s. Introduce tstart (s): ", globalTimestamp/1000000000);
	scanf("%d", &curr_time);
	tstart = (long long)curr_time * 1000000000; // back to nanoseconds
		
	propose(&descriptor, elist, tstart, AGREEMENT_TYPE , value, &outp); //call TTCB
	if (outp.error < 0) {
	  print_error_TTCB(outp.error);
	  goto finalize;
	}
	// decide value
        do {
	  decide(&descriptor, outp.tag, &outd); //call TTCB
	  usleep(300);
	  
        } while (outd.error == AGREEMENT_RUNNING);
	
	if (outd.error < 0) {
	  print_error_TTCB(outd.error);
	  goto finalize;
	}
	printf("Decide ok. Value decided: ");
	printResult(outd.value);
	printf("\nproposed-ok: ");
	printMask(outd.mask1, size_elist);
	printf("  proposed-any: ");
	printMask(outd.mask2, size_elist);
	printf("\n");
	
 finalize:
        printf("Going to close connection!\n");
        closeTTCBConnection(&descriptor, &err_out); //call TTCB
	printf("Ending...\n\n");
	return 0;
}
