
/*
 *	RTskbuff.h
 *	based on <include/linux/skbuff.h>	
 *
 *	RT-INET 1.0
 *
 *	(C)2002, Navigators Group	<www.navigators.di.fc.ul.pt>
 *		 Pedro Martins		<pmartins@di.fc.ul.pt>
 *	Real-Time Socket Buffers	
 *
 * 	Last Review Date: XX
*/
#ifndef	_RTskbuff
#define _RTskbuff

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/errno.h>
#include <linux/in.h>
#include <linux/ioport.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/mii.h>
#include <linux/init.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/ethtool.h>
#include <linux/highmem.h>
#include <asm/irq.h>			
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#include "mem_mng.h"
#include <rtai.h>

extern object_list	RTskbuffs;

#define SKBUFF_PKT_SIZE	SKB_DATA_ALIGN(1600)
 
 typedef struct RTsk_buff{
 	/* These two members must be first. */
//	struct RTsk_buff	* next;			/* Next buffer in list 				*/
//	struct RTsk_buff	* prev;			/* Previous buffer in list 			*/

//	struct sk_buff_head * list;		/* List we are on				*/
//	struct sock	*sk;			/* Socket we are owned by 			*/
	RTIME		timestamp;		/* Time we arrived				*/
	struct RTnet_device	*dev;		/* Device we arrived on/are leaving by		*/

	/* Transport layer header */
	union
	{
//		struct tcphdr	*th;
//		struct udphdr	*uh;
//		struct icmphdr	*icmph;
//		struct igmphdr	*igmph;
//		struct iphdr	*ipiph;
//		struct spxhdr	*spxh;
		unsigned char	*raw;
	} h;

	/* Network layer header */
	union
	{
//		struct iphdr	*iph;
//		struct ipv6hdr	*ipv6h;
//		struct arphdr	*arph;
//		struct ipxhdr	*ipxh;
		unsigned char	*raw;
	} nh;
  
	/* Link layer header */
	union 
	{	
	  	struct ethhdr	*ethernet;
	  	unsigned char 	*raw;
	} mac;

//	struct  dst_entry *dst;   //O que e?

	/* 
	 * This is the control buffer. It is free to use for every
	 * layer. Please put your private variables there. If you
	 * want to keep them across layers you have to do a skb_clone()
	 * first. This is owned by whoever has the skb queued ATM.
	 */ 
//	char		cb[48];	 

	unsigned int 	len;			/* Length of actual data			*/
 	unsigned int 	data_len;
//	unsigned int	csum;			/* Checksum 					*/
	unsigned char 
//			__unused,		/* Dead field, may be reused			*/
//			cloned, 		/* head may be cloned (check refcnt to be sure). */
  			pkt_type,		/* Packet class					*/
  			ip_summed;		/* Driver fed us an IP checksum			*/
	__u32		priority;		/* Packet queueing priority			*/
//	atomic_t	users;			/* User count - see datagram.c,tcp.c 		*/
	unsigned short	protocol;		/* Packet protocol from driver. 		*/
//	unsigned short	security;		/* Security level of packet			*/
	unsigned int	truesize;		/* Buffer size 					*/

//RTIP hack for cloned buffers
//	struct RTsk_buff * clone_skb;		

	unsigned char	*head;			/* Head of buffer 				*/
	unsigned char	bdata[SKBUFF_PKT_SIZE];	/* Data Buffer					*/
	unsigned char   *data;
	unsigned char	*tail;			/* Tail pointer					*/
	unsigned char 	*end;			/* End pointer					*/

	pt_obj_list	obj_list;

//	void 		(*destructor)(struct sk_buff *);	/* Destruct function		*/
			
} RTsk_buff;
 

/**
 *	RTalloc_skb	-	allocate a network buffer
 *	@size: size to allocate
 *
 *	Allocate a new &RTsk_buff. The returned buffer has no headroom and a
 *	tail room of size bytes. The object has a reference count of one.
 *	The return is the buffer. On a failure the return is %NULL.
 *
 */
 
extern struct RTsk_buff *RTalloc_skb(void);
extern struct RTsk_buff *RTskb_copy(struct RTsk_buff* skb);

/**
 *	RTskb_reserve - adjust headroom
 *	@skb: buffer to alter
 *	@len: bytes to move
 *
 *	Increase the headroom of an empty &sk_buff by reducing the tail
 *	room. This is only allowed for an empty buffer.
 */

static inline void RTskb_reserve(struct RTsk_buff *skb, unsigned int len)
{
	skb->data+=len;
	skb->tail+=len;
}

/**
 *	RTdev_alloc_skb - allocate an skbuff for sending
 *	@length: length to allocate
 *
 *	Allocate a new &sk_buff and assign it a usage count of one. The
 *	buffer has unspecified headroom built in. Users should allocate
 *	the headroom they think they need without accounting for the
 *	built in space. The built in space is used for optimisations.
 *
 *	%NULL is returned in there is no free memory. Although this function
 *	allocates memory it can be called from an interrupt.
 */
 
static inline struct RTsk_buff *RTdev_alloc_skb(void)
{
	struct RTsk_buff *skb;

	skb = RTalloc_skb();
	if (skb)
		RTskb_reserve(skb,16);	
	return skb;
}

/**
 *	RTkfree_skb - free an sk_buff
 *	@skb: buffer to free
 *
 *	Drop a reference to the buffer and free it if the usage count has
 *	hit zero.
 */
 
static inline void RTkfree_skb(struct RTsk_buff *skb)
{
	//if (skb->cloned==1){
	//	atomic_dec(&(skb->clone_skb->users));
	//	RTkfree_skb(skb->clone_skb);
	//}

	//if (atomic_read(&skb->users) == 1)
	free_obj_list_item(&RTskbuffs,skb->obj_list);		
}

#define RTdev_kfree_skb(x)	RTkfree_skb(x)   

//VER MELHOR
#define RTdev_kfree_skb_irq(x)	RTkfree_skb(x)   


/**
 *	RTskb_put - add data to a buffer
 *	@skb: buffer to use 
 *	@len: amount of data to add
 *
 *	This function extends the used data area of the buffer. If this would
 *	exceed the total buffer size the kernel will panic. A pointer to the
 *	first byte of the extra data is returned.
 */
 
static inline unsigned char *RTskb_put(struct RTsk_buff *skb, unsigned int len)
{
	unsigned char *tmp=skb->tail;
	skb->tail+=len;
	skb->len+=len;
	if(skb->tail>skb->end) {
		return NULL;
	}
	return tmp;
}

static inline unsigned char *__RTskb_put(struct RTsk_buff *skb, unsigned int len)
{
	unsigned char *tmp=skb->tail;
	skb->tail+=len;
	skb->len+=len;

	return tmp;
}

static inline unsigned char *__RTskb_push(struct RTsk_buff *skb, unsigned int len)
{
	skb->data-=len;
	skb->len+=len;
	return skb->data;
}

/**
 *	RTskb_push - add data to the start of a buffer
 *	@skb: buffer to use 
 *	@len: amount of data to add
 *
 *	This function extends the used data area of the buffer at the buffer
 *	start. If this would exceed the total buffer headroom the kernel will
 *	panic. A pointer to the first byte of the extra data is returned.
 */

static inline unsigned char *RTskb_push(struct RTsk_buff *skb, unsigned int len)
{
	skb->data-=len;
	skb->len+=len;
	if(skb->data<skb->head) {
		return NULL;
	}
	return skb->data;
}


/**
 *	RTskb_tailroom - bytes at buffer end
 *	@skb: buffer to check
 *
 *	Return the number of bytes of free space at the tail of an sk_buff
 */

static inline int RTskb_tailroom(const struct RTsk_buff *skb)
{
	return skb->end-skb->tail;
}

/**
 *	RTskb_headroom - bytes at buffer head
 *	@skb: buffer to check
 *
 *	Return the number of bytes of free space at the head of an &sk_buff.
 */
 
static inline int RTskb_headroom(const struct RTsk_buff *skb)
{
	return skb->data-skb->head;
}


static inline char *__RTskb_pull(struct RTsk_buff *skb, unsigned int len)
{
	skb->len-=len;
	if (skb->len < skb->data_len)
		return NULL;
	return 	skb->data+=len;
}

/**
 *	RTskb_pull - remove data from the start of a buffer
 *	@skb: buffer to use 
 *	@len: amount of data to remove
 *
 *	This function removes data from the start of a buffer, returning
 *	the memory to the headroom. A pointer to the next data in the buffer
 *	is returned. Once the data has been pulled future pushes will overwrite
 *	the old data.
 */

static inline unsigned char * RTskb_pull(struct RTsk_buff *skb, unsigned int len)
{	
	if (len > skb->len)
		return NULL;
	return __RTskb_pull(skb,len);
}

/*
  RTIP does not support fragmented sk_buffs.
  However, it defines a dummy RTskb_shinfo macro to provide an
  API for device drivers consistent with Linux.	
*/

static struct skb_shared_info dummy_skbshared;
#define RTskb_shinfo(SKB)	((struct skb_shared_info*)&dummy_skbshared)


extern int initRTskbuffs(int n_buffs);
extern void endRTskbuffs(void);

#endif
