#ifndef SHARE_TCB
#define SHARE_TCB
#include <rtai.h>
#include "tcb/system_defs.h"

#define MAX_TCB_PORT 100

/*
+--------------------------------+
| ERROS DEVOLVIDOS AS APLICACOES | 
+--------------------------------+
*/

#define	OK			 0
#define TCB_ACESS_ERROR		-1 	// Erro no acesso a tcb         
#define NO_AVAIL_CONN		-2      // Conexao indisponivel        
#define NO_AVAIL_LOCAL_MEASURE  -3      // medicao local nao disponivel 
#define INVALID_LOCALM_DESC	-4      // descriptor de medicao local invalido 
#define NO_AVAIL_EXEC		-5      // execucao nao disponivel 
#define FUNC_NOT_FOUND		-6      // funcao nao encontrada   
#define WCET_FUNC_NOT_FOUND	-7      // funcao wcet nao encontrada
#define CANT_RUN_TIMELY		-8      // nao consegue correr atempadamente 
#define NO_AVAIL_LOCAL_FD	-9      // detector de falhas local nao disponivel 
#define ALREADY_FAILED          -10     // a tcb detectou que a falha deu-se antes de 
				        // processar o pedido 		  	 
#define INVALID_LOCALFD_DESC	-11     // descriptor de deteccao de falhas locais
#define TOO_MANY_DISTRIBUTED_FD  -12     // demasiadas deteccoes simultaneas
#define BURSTY_SENDS		-13	// demasiados sends no round
#define IN_INIT_STATE		-14	// a tcb encontra-se em fase de inicializacao
#define HANDLER_EXISTS		-15	// o handler remoto ja existe
#define NO_AVAIL_RESOURCES	-16     // recursos insuficientes para processar o pedido
#define BAD_PORT		-17     // porto invalido
#define ALREADY_BOUNDED		-18	// o canao de QoS ja foi aberto
#define NOT_BOUNDED		-19	// a aplicacao nao detem o canal de QoS

/*
+--------------------------------------------+
| SERVICOS FORNECIDOS PELA TCB AS APLICACOES |
+--------------------------------------------+
*/

typedef 
  enum {
          get_connection, 	        // pedido de conexao a tcb             	
          end_connection,		// final de conexao
          get_kernel_info,		// informacao do kernel (T)TCB	
          get_timestamp,		// obter estampilha temporal
          get_global_timestamp,		// obter estampilha temporal global
          get_local_global_timestamp,	// obter estampilha local+global 
          get_id,			// obter id da TCB local
          start_localM,			// iniciar medicao local
          end_localM,			// terminar medicao local	
          exec,				// execucao local 
          start_localFD,		// iniciar detecao de falha local
          end_localFD,			// terminar deteccao de falha local
          start_distributedFD,		// iniciar detecao de falhas distribuidas
          start_distributedRemoteFD,	// iniciar detecao remota de falhas distribuidas	
	  set_RemoteHandler,          	// 
          msg_received_on_payload,	// recepcao de mensagem no canal de PayLoad
          mon_distributed,	
          set_coverage,	
          winfo_to_fifo,		// especifica para que fifo deve ser enviada a informacao
          				// relacionada com um dado porto de comunicacoes
          stop_winfo_port		// para de enviar winfo respectiva a um porto dado		          	
#ifdef TTCB          
          ,get_timestamp_TTCB_enum,              // obter estampilha temporal sincronizada
          localAuthentication_enum,
          end_connection_TTCB_enum,
          propose_enum,
          decide_enum,
          getRandom_enum,
          getLocalPublicKey_enum,
          getLocalPublicKey_S_enum,
          getLocalTTCBid_enum,
          crashedLocalTTCBid_enum,
          crashedLocalTTCBeid_enum

#endif
       } 
Tservice;



/*
+----------------------------------------+
| CABECALHO DAS MENSAGENS ENVIADAS A TCB |
+----------------------------------------+
*/

typedef struct  {
	Tservice 	srvc;		// identificacao do servico (Tservice)
	int		sizeofmsg;      // tamanho dos parametros do servico
} ReqTcbHeader;


/*
+----------------------+
| FORMATO DAS DURACOES |
+----------------------+
*/

typedef long long TCBtimeinterval; 



/*
+---------------------+
| ESTAMPILHA TEMPORAL |
+---------------------+
*/

typedef long long TCBtimestamp;

typedef struct{
		TCBtimestamp ts;
		TCBtimestamp gts;	
	} 
local_global_ts_t;


/*
+----------------------------------------------------------------------------+
| INFORMACAO DE RETORNO DO SERVICO 'end_localM'- passada pela TCB a aplicacao|
|      						 (bibl.TCB) no fifo          |
+----------------------------------------------------------------------------+
*/

typedef struct 
        {
          char 		  stat;         // ok ou codigo erro 
          TCBtimeinterval tm;		// duracao 
          TCBtimestamp    ts;		// instante final da medicao
        }
endLocalMResult;        



/*
+-----------------------------------------------------------------------------+
| IDENTIFICACAO DE UMA FUNCAO DEFINIDA NO NUCLEO DO LINUX (funcoes que definem|
| handlers invocados aquando de falhas temporais,funcoes de execucao atempada)|
| Estrutura usada pelas aplicacoes ao aceder aos servicos fornecidos pela     |      
| bliblioteca TCB.							      |									       
+-----------------------------------------------------------------------------+
*/

typedef struct {
	char  module[21];                  // nome do modulo onde esta definida a funcao
	char  funcname[21];		// nome da funcao a executar
	char  wcetname[21];		// nome da funcao que especifica o WCET
}  
Func_Id;                                



/*
+----------------------------------------------------------------------------+
|  PARAMETROS DO SERVICO 'exec' - passados pela aplicacao (biblioteca TCB) a |
|                                 TCB num fifo.				     |
+----------------------------------------------------------------------------+
*/


typedef struct 
	{
	  TCBtimestamp	  start_ev;     // inicio temporal da execucao
	  TCBtimeinterval delay;        // offset da execucao a partir de start_ev
	  TCBtimeinterval max_exec;     // tempo maximo para o fim da execucao a partir de start_ev
	  char 		  modName[21];  // nome do modulo onde esta definida a funcao
	  char		  funcName[21]; // nome da funcao a executar
 	  char 		  wcetName[21]; // nome da funcao que especifica o WCET
 	  char 		  TraceName[21]; // identificacao da funcao para tracing
	}
execPars;



/*
+-----------------------------------------------------------------------------+
| PARAMETROS DO SERVICO 'start_localFD'-passados pela aplicacao(biblioteca TCB)|
|			               a TCB num fifo                         |
+-----------------------------------------------------------------------------+
*/

typedef struct 
	{
	  TCBtimestamp	  start_ev;     // inicio temporal da deteccao
	  TCBtimeinterval spec;         // especificacao do intervalo temporal 
	  				// para a geracao de falha
	  TCBtimeinterval deadline;     // especificacao do deadline 				
	  char 		  modName[21];  // nome do modulo onde esta definida a funcao
	  char		  funcName[21]; // nome da funcao a executar
 	  char 		  wcetName[21]; // nome da funcao que especifica o WCET
	}
localFDPars;


/*
+------------------------------------------------------------------------------------+
| PARAMETROS DO SERVICO 'start_distributedFD'-passados pela aplicacao(biblioteca TCB)|
|			               a TCB num fifo                                |
+------------------------------------------------------------------------------------+
*/

typedef struct 
	{
	  TCBtimestamp	  start_ev;     // inicio temporal da deteccao
	  TCBtimeinterval spec;         // especificacao do intervalo temporal 
	  				// para a geracao de falha
	  TCBtimeinterval deadline;     // especificacao do deadline 								
	  unsigned char	  to;						
	  unsigned int	  destination_port;
	  char 		  modName[21];  // nome do modulo onde esta definida a funcao
	  char		  funcName[21]; // nome da funcao a executar
 	  char 		  wcetName[21]; // nome da funcao que especifica o WCET
	}
distributedFDPars;	

/*
+------------------------------------------+
| PARAMETROS DO SERVICO 'set_RemoteHandler'|
+------------------------------------------+
*/

typedef struct 
	{
	  TCBtimeinterval deadline;     // especificacao do deadline 								
	  unsigned int	  destination_port;
	  char 		  modName[21];  // nome do modulo onde esta definida a funcao
	  char		  funcName[21]; // nome da funcao a executar
 	  char 		  wcetName[21]; // nome da funcao que especifica o WCET
	}
setRemoteHandlerPars;	



/*
+-----------------------------------------------------------------------------+
| INFORMACAO DE RETORNO DO SERVICO 'end_localFD'- passada pela TCB a aplicacao|
|      						 (bibl.TCB) no fifo           |
+-----------------------------------------------------------------------------+
*/

typedef struct
	{
	  int             stat;	        // ok ou codigo erro 
	  TCBtimestamp    end_ev;       // instante da desactivacao da deteccao
	  TCBtimeinterval procdelay;    // intervalo temporal
	  char		  faulty;       // houve ou nao falha 
	}
endLocalFDResult;



typedef struct
	{
		unsigned long seq;
		unsigned long long time;
		unsigned char from;
	}
msg_rcv_on_payloadPars;	

typedef struct 
	{
		unsigned char fail;   // =2 means no info
		unsigned char lateness_avail;	
		unsigned long lateness_degree;
	}
fail_info;

typedef struct
	{
		unsigned  char broadcast;
		unsigned  char from;
		unsigned  long seq;
		unsigned  int  port;
		fail_info failure_info[NUMBER_OF_MACHINES];
	}
fail_info_to_app;		


typedef struct
	{
		unsigned char		srcId;
		unsigned long long      gstartInst;  	// global start ts
		unsigned long		seq;  	        // 0 means no info
		unsigned int	        destination_port;
	}
distAction_id_t;	

#ifndef TTCB
typedef struct{
		int desc_orig;
		int wi_orig;	
	}
descriptor_t;
#endif

/*
------------------
(T)TCB kernel info
------------------
*/

typedef struct{
	int version; 		//kernel version
	int sub_version;	//sub version
	int ttcb_extentions;	//TTCB enabled?

// TO BE EXTENDED !!!
}
tcb_kernel_info_t;


/*
-------------
DEBUG COMANDS
-------------
*/

#define DUMP_INTERNAL_STATE	0
#define BENCH_RESULTS		1



/*
+-----------------------------------------------------------------------+
|  PARAMETROS DO SERVICO 'monDistributed' 				|	
+ ----------------------------------------------------------------------+
*/

typedef enum {coverage_awareness,coverage_stability} QoSAdaptationMode_t;

typedef struct 
	{
	  unsigned long	  bound;
	  double	  coverage;
	  QoSAdaptationMode_t mode;
	  unsigned long	  dev;	
	  unsigned int    port;
	  char 		  modName[21];  // nome do modulo onde esta definida a funcao
	  char		  funcName[21]; // nome da funcao a executar
 	  char 		  wcetName[21]; // nome da funcao que especifica o WCET
	}
monDistPars;

typedef struct 
	{
	  double	  coverage;
	  unsigned int    port;
	}
setCoveragePars;

#endif


