// TTCB Agreement Service test application

import java.lang.*;
import java.io.*;
import ttcb.TTCB;      // TTCB package
import ttcb.descriptor_t;
import ttcb.timestamp_out_t;
import ttcb.propose_out_t;
import ttcb.decide_out_t;
import ttcb.share_app_ttcb;

class Agreement extends share_app_ttcb {

    public static void printHex(byte b[], int len) // print array of bytes in hex
    {                                            // based in code in sourceforge.net
        int i;
        for (i=0; i<len; i++)
            System.out.print(""+"0123456789ABCDEF".charAt(0xf&b[i]>>4)+"0123456789ABCDEF".charAt(b[i]&0xF) + " ");
    }


    public static void printBoolean(boolean b[], int len) //print array of booleans
    {
        int i;
        for (i=0; i<len; i++)
	    if (b[i] == false)
                System.out.print("0");
	    else
                System.out.print("1");
    }


    public static void main(String[] args) throws IOException {

    	TTCB my_ttcb = new TTCB();  // create a TTCB interface
	short myEid, eid;
        descriptor_t descriptor = new descriptor_t();
	timestamp_out_t tsOut = new timestamp_out_t();
	propose_out_t proposeOut;
	decide_out_t decideOut;
	BufferedReader stdin=
	    new BufferedReader(new InputStreamReader(System.in));
	String s, s1;
	short elist[] = new short[MAX_SIZE_ELIST];
	byte value[] = new byte[SIZE_VALUE];
	long tstart;
	int i;
	int size_elist;
	long begin, end;
	int err = 0;

	// ----- init -----
	try {
            err = my_ttcb.openTCBConnection(descriptor);
	} catch(Exception ex){ex.printStackTrace(); System.exit(1);}
	if(err < 0) {
	    System.out.println("Error opening connection with the TTCB: " + err);
	    System.exit(1);
	}
	System.out.println("My eid is: " + descriptor.eid);
	
	// ----- propose -----
	System.out.println("Please introduce the elist (one eid at a time and 0 to terminate):");
	i = 0;
	do {
	    s = stdin.readLine();
	    s1 = s.trim();
	    try{ 
		eid = new Short(s1).shortValue(); 
	  	elist[i] = eid;
		i = i + 1;		
	    }
	    catch(NumberFormatException ex){eid = 1;}
	} 
	while (eid != 0 && i < MAX_SIZE_ELIST-1);
	elist[i-1] = END_ELIST;
	size_elist = i-1;

	System.out.println("Introduce the value to propose (0 to 127):");
	s = stdin.readLine();
        s1 = s.trim();
	value[0] = new Byte(s1).byteValue();

	err = my_ttcb.getTCBGlobalTimestamp(descriptor,tsOut);
	if (err != 0) {
	    System.out.println("Error in getTTCBTimestamp." + err);
	    System.exit(1);
	}

	System.out.print("Current time is " + (tsOut.ts/1000000000) + ". Introduce tstart (s): ");
        s = stdin.readLine();
        s1 = s.trim();
        tstart = new Long(s1).longValue() * 1000000000;

	begin = System.currentTimeMillis();

	proposeOut = my_ttcb.propose(descriptor, elist, tstart, TTCB_TBA_RMULTICAST, value);
	if(proposeOut == null) {
	    System.out.println("Error in propose (null returned)");
	    System.exit(1);
	}
	if (proposeOut.error != 0) {
	    System.out.println("Error in propose. " + proposeOut.error);
	    System.exit(1);
	}

	// ----- decide -----
	do {
	    decideOut = my_ttcb.decide(descriptor, proposeOut.tag);
	    if(proposeOut == null) {
	        System.out.println("Error in propose (null returned)");
	        System.exit(1);
	    }
	} while (decideOut.error < 0);

	end = System.currentTimeMillis();

	System.out.println("Decide ok. Value decided: " + decideOut.value[0]);
	System.out.print("Entities that proposed the value decided (proposed-ok): ");
	printBoolean(decideOut.mask1, size_elist);
	System.out.println("");
	System.out.print("Entities that proposed any value (proposed-any): ");
	printBoolean(decideOut.mask2, size_elist);
	System.out.println("");

	System.out.println("Agreement delay: " + (end - begin) + " msec");

	// ----- end -----
	my_ttcb.closeTCBConnection(descriptor);

	System.exit(0);

    }
}
