/*	 
	(T)TCB user C library 

--[TTCB 1.1]----------------------------------------------------

    (Trusted) Timely Computing Base 1.1
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
    ------------------------------------------------------------   
    
    Pedro Martins <pmartins@di.fc.ul.pt>
    	tcb services
    
    Miguel Correia <mpc@di.fc.ul.pt>
    	ttcb services
    
    Nuno Miguel Neves <nnveves@di.fc.ul.pt>
    	ttcb services
     

-----------------------------------------------------------------	
*/


#include "tcb/tcb.h"
#include "tcb/system_defs.h"
#define tcb

static int __this_tcb_id = -1;

#ifdef TTCB
#include "tcb/share_app_ttcb.h"
int getLocalPublicKey(descriptor_t * descriptor);
int localAuthentication( descriptor_t * descriptor);

void call_TTCB(Tservice srvc, descriptor_t *descriptor, in_hdr_t *in, out_hdr_t *out, int size_in, int size_out);
void closeTTCBConnection(descriptor_t *descriptor, error_out_t *out);
void print_error_TTCB(int err);


void print_mpi(char * ann,MPI a)
{
  int i=0;
  int n=0;
  int limbs=0;


  limbs=mpi_get_nbits(a);


  printf("\n%s :  ", ann);

  if( a->sign )
    printf("%c",'-');
 #define X "8"

  for(i=a->nlimbs; i > 0 ; i-- ) {
    n += printf(i!=a->nlimbs? "%0" X "lX":"%lX", (ulong)a->d[i-1]);
  }
#undef X  
  if( !a->nlimbs )
    printf("%c",'0');
  
  printf("\n");

  return;
}

void get_random(char *p, int length)
{
  int fd;
  int chread;

  fd=open("/dev/urandom",0);
  if(fd<0) perror("ERROR:  ");
  chread = read(fd, p, length);
  if (chread <0) perror("ERROR: ");
  close(fd);
  return;

}

#endif //TTCB

//=================================
//Security Wrapper for TCB services
//=================================
int call_tcb(Tservice service, descriptor_t *descriptor, void * in, void *out, int size_in, int size_out ){

#ifdef TTCB
	tcb_service_big_in tempbigin;
	tcb_service_in * tempin = (tcb_service_in*)&(tempbigin);
	int st_size=sizeof(tcb_service_in);
	tcb_service_out tempout;

	if (size_in>0) {	
		memcpy(tempin->buffer,in,size_in);
		if (size_in+sizeof(in_hdr_t)>sizeof(tcb_service_in))
			st_size = sizeof(tcb_service_big_in);
	}

	call_TTCB(service, descriptor, (in_hdr_t *)tempin, (out_hdr_t *)&tempout, st_size,sizeof(tempout));

	if (tempout.header_out.error<0)
	  {
		return tempout.header_out.error;
	  }

	if (size_out!=0) 
		memcpy(out,tempout.buffer,size_out);

	return 0;
#else
	
	char msg [sizeof(ReqTcbHeader)+size_in];
	ReqTcbHeader * header_pt = (ReqTcbHeader*)&msg;
 
	header_pt->srvc = service;
	header_pt->sizeofmsg = size_in;

	if( size_in > 0)
		memcpy(msg+sizeof(ReqTcbHeader), in, size_in);

	if (write(descriptor->desc_orig, &msg, sizeof(msg)) < 0) 
	 	return TCB_ACESS_ERROR;
	
	if (read(descriptor->desc_orig,out,size_out) <0 )
		return TCB_ACESS_ERROR;
	
	return 0;
#endif	
}


//=================================
// TCB services
//=================================

/*
==========================================================================

 returns descriptor
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   IN_INIT_STATE
	   NO_AVAIL_CONN
 TTCB:
	  INVALID_PROTECTION
	  FAILED_LOCAL_AUTHENTICATION	
*/

int openTCBConnection(descriptor_t *tcb_desc)
{
	int fd,con;
	ReqTcbHeader msg;
	char aux[30];
	int error;
#ifdef TTCB
	int a;
#endif
	if ((fd=open("/dev/rtf1", O_RDWR))==-1)   
		return TCB_ACESS_ERROR;
	
	msg.srvc=get_connection;
	msg.sizeofmsg=0;
	
	if (write(fd, &msg, sizeof(msg)) < 0) 
		return TCB_ACESS_ERROR;
	
	if (read(fd,&con,sizeof(int)) <0 )
		return TCB_ACESS_ERROR;
		
	close(fd);	
		
	if (con<0) return con;	
	
	sprintf(aux,"/dev/rtf%d",con);
	if ( (tcb_desc->desc_orig = open(aux, O_RDWR)) < 0) 
		return TCB_ACESS_ERROR;
		
	sprintf(aux,"/dev/rtf%d",(con+MAX_TCB_CONNECTIONS));  

	if ((tcb_desc->wi_orig = open(aux, O_RDWR)) < 0) 
		{
		msg.srvc=end_connection;
		msg.sizeofmsg=0;
	
		if (write(tcb_desc->desc_orig, &msg,sizeof(msg)) < 0) 
			return TCB_ACESS_ERROR;
	
		close(tcb_desc->desc_orig);	
		return TCB_ACESS_ERROR;	
		}
	
#ifdef TTCB
	if ((tcb_desc->TTCB_PublicKey).e == NULL)
	  {
	    // GET TTCB Public Key; 
	    error=getLocalPublicKey(tcb_desc);

	    if (error) {
	    closeTCBConnection(tcb_desc);	     
	    return error;
	    }
	  }
	
	for(a=0;a<16 && tcb_desc->secret[a]==0;a++){};
	/* If a reaches 16, then security is not defined by the application.
	 * We assume then to use no protection to access TTCB and 
	 * randomly choose the key and the challenge.
	 */

	if(a==16)
	  {
	    get_random((char*)&(tcb_desc->secret), 16);
	    get_random((char*)&(tcb_desc->challenge), 8);
	    tcb_desc->protection = PROTECTION_NONE;
	  }
	error=localAuthentication(tcb_desc);
	if (error)
	  {
	    closeTCBConnection(tcb_desc);
	    return error;
	  }
#else
	return 0;
#endif
}


/*
==========================================================================

 returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/


int closeTCBConnection(descriptor_t *descriptor)
{
#ifdef TTCB
  error_out_t out;
  closeTTCBConnection(descriptor, &out);
  return out.error;    
#else
	ReqTcbHeader msg;	
	
	msg.srvc=end_connection;
	msg.sizeofmsg=0;
	
	if (write(descriptor->desc_orig, &msg,sizeof(msg)) < 0) 
		return TCB_ACESS_ERROR;
	
	close(descriptor->desc_orig);	
	close(descriptor->wi_orig);    
	return 0;
#endif
}

/*
==========================================================================

 OK - returns a timestamp 
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int getTCBTimestamp(descriptor_t *descriptor, TCBtimestamp * ts)
{
	int ret;
	
	ret = call_tcb(get_timestamp, descriptor, NULL, (TCBtimestamp*)ts, 0, sizeof(TCBtimestamp)) ;

	if (ret) return ret;
	return 0;	
}

/*
==========================================================================

 OK - returns the kernel info in *kern_inf
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int getTCBKernelInfo(descriptor_t * descriptor,tcb_kernel_info_t* kern_inf){
	
	int ret;

	ret = call_tcb(get_kernel_info, descriptor, NULL, (tcb_kernel_info_t*)kern_inf, 0, sizeof(tcb_kernel_info_t)) ;

	if (ret) return ret;
	return 0;	
}


#ifdef DISTRIBUTED
/*
==========================================================================

 OK - returns a global timestamp
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/
int getTCBGlobalTimestamp(descriptor_t * descriptor, TCBtimestamp * gts)
{
	int ret;
	
	ret = call_tcb(get_global_timestamp, descriptor, NULL, (TCBtimestamp*)gts, 0, sizeof(TCBtimestamp)) ;

	if (ret) return ret;
	return 0;	
}

/*
==========================================================================

 OK - returns a related local/global timestamp 
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int getTCBLocalAndGlobalTimestamp(descriptor_t * descriptor, TCBtimestamp * ts, TCBtimestamp * gts)
{
	int ret;
	local_global_ts_t result;
	
	ret = call_tcb(get_local_global_timestamp, descriptor, NULL, (local_global_ts_t*)&result, 0, sizeof(local_global_ts_t));

	*ts=result.ts;
	*gts=result.gts;

	if (ret) return ret;		

	return 0;	
}


#endif

/*
==========================================================================

 returns the id of the local TCB
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/
int getTCBid(descriptor_t * descriptor) {
	int tid;
	int ret;
	
	if (__this_tcb_id!=-1){
		return __this_tcb_id;
	}

	ret = call_tcb(get_id, descriptor, NULL, (int*)&tid, 0, sizeof(int)) ;

	if (ret) return ret;
	__this_tcb_id = tid;
	return tid;
}

#ifdef LOCAL_MEASURE
/*
==========================================================================


//NOTA: a medicao local vai ser passada para user level....nao ha necessidade
//	de ser concretizada dentro da TCB, mas para ja fica assim...

 returns local measurement descriptor
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   NO_AVAIL_LOCAL_MEASURE   	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/
int startTCBMeasurement(descriptor_t * descriptor,TCBtimestamp ts)
{
	int ret;
        int desc;
	
	ret = call_tcb(start_localM, descriptor, (TCBtimestamp*)&ts, (int*)&desc, sizeof(TCBtimestamp), sizeof(int));
	if (ret) return ret;

	return desc;
}

#endif //LOCAL_MEASURE

#ifdef LOCAL_MEASURE
/*
==========================================================================

 returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   INVALID_LOCALM_DESC	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int endTCBMeasurement(descriptor_t * descriptor, int measuredesc, TCBtimeinterval* tm, TCBtimestamp* end_ev)
{	
	endLocalMResult res;		
	int ret;
		
	ret = call_tcb(end_localM, descriptor, (int*)&measuredesc, (endLocalMResult*)&res, sizeof(int), sizeof(endLocalMResult));

	if (ret) return ret;

	if (res.stat!=OK) return res.stat;	
		
	*tm=res.tm;
	*end_ev=res.ts;	

	return OK;		
}

#endif //LOCAL_MEASURE
/*

==========================================================================

 returns OK - end timestamp
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   FUNC_NOT_FOUND	  
	   WCET_FUNC_NOT_FOUND	  			
	   NO_AVAIL_EXEC          
	   CANT_RUN_TIMELY
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

#ifdef LOCAL_TIMELY

int startTCBExec(descriptor_t * descriptor,TCBtimestamp start_ev, TCBtimeinterval delay, TCBtimeinterval max_exec,Func_Id func,char* TraceName,TCBtimestamp * end_ts)
{
	execPars pars;	
	int ret;
	long long res;
	
	pars.start_ev=start_ev;
	pars.delay=delay;
	pars.max_exec=max_exec;
	
	strncpy(pars.funcName,func.funcname,21);
	strncpy(pars.modName,func.module,21);
	strncpy(pars.wcetName,func.wcetname,21);
	strncpy(pars.TraceName,TraceName,21);

	ret = call_tcb(exec, descriptor, (execPars*)&pars, (long long*)&res, sizeof(execPars), sizeof(TCBtimestamp));	

       	if (ret) return ret;
       
        if (res < 0)
		return res;		
	
	*end_ts = (TCBtimestamp)res;
       
	return 0;	
}

#endif //LOCAL_TIMELY

#ifdef LOCAL_FAULT
/*
==========================================================================

 returns OK - local failure detection descriptor
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   FUNC_NOT_FOUND
	   WCET_FUNC_NOT_FOUND	  			
           NO_AVAIL_LOCAL_FD
	   ALREADY_FAILED   
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/
int startTCBLocalFD(descriptor_t * descriptor,TCBtimestamp start_ev, TCBtimeinterval spec, TCBtimeinterval deadline, Func_Id func)
{
	localFDPars pars;	
	int res, ret;
		
	pars.start_ev=start_ev;
	pars.spec=spec;
	pars.deadline=deadline;
	
	strncpy(pars.funcName,func.funcname,21);
	strncpy(pars.modName,func.module,21);
	strncpy(pars.wcetName,func.wcetname,21);
													
	ret = call_tcb(start_localFD, descriptor, (localFDPars*)&pars, (int*)&res, sizeof(localFDPars), sizeof(int));	

        if (ret) return ret;
       
	return res;	
}


/*
==========================================================================

 returns OK 
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   INVALID_LOCALFD_DESC	   
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/
int endTCBLocalFD(descriptor_t * descriptor,int fddesc, TCBtimestamp * end_ev, TCBtimeinterval * procdelay, char * faulty)
{
	int ret;
	endLocalFDResult res;	

	ret = call_tcb (end_localFD, descriptor, (int*)&fddesc, (endLocalFDResult*)&res, sizeof(int), sizeof(endLocalFDResult));	
       	
        
       
	if (res.stat!=OK) return res.stat;	
		
	*end_ev=res.end_ev;
	*faulty=res.faulty;
	*procdelay=res.procdelay;

	return OK;			
}

#endif //LOCAL_FAULT

/*
==========================================================================

returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

#ifdef DIST_MEASURE

int startDistMeasurement(descriptor_t * descriptor, TCBtimestamp send_evt, distAction_id_t *actionid){	
//NOTA: send_evt e uma estampilha global
	int ret;

	actionid->gstartInst = send_evt;

	ret = getTCBid(descriptor);
	
	if (ret<0) return ret;

	actionid->srcId = ret;
	actionid->seq = 0;

	return OK;	
}

/*
==========================================================================

returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int endDistMeasurement(descriptor_t * descriptor, distAction_id_t *actionid, TCBtimestamp * local_rcv_time, TCBtimestamp * global_rcv_time, TCBtimeinterval * procdelay){	
	int ret;

	ret = getTCBLocalAndGlobalTimestamp(descriptor,local_rcv_time,global_rcv_time);
	
	if (ret<0) return ret;

	*procdelay=*global_rcv_time - actionid->gstartInst;

	return OK;	
}
#endif

/*
==========================================================================

returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
	   BURSTY_SENDS
	   FUNC_NOT_FOUND	
	   WCET_FUNC_NOT_FOUND
	   TOO_MANY_DISTRIBUTED_FD
           ALREADY_FAILED
           NO_AVAIL_EXEC
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	

//spec in us !!!!  
*/

#ifdef DIST_FAULT

int startDistFD(descriptor_t * descriptor,unsigned char dest_mask,unsigned int dest_port,TCBtimestamp send_evt,TCBtimeinterval spec,TCBtimeinterval deadline,Func_Id func, distAction_id_t *actionid)
{
//NOTA: send_evt e uma estampilha global
	int ret;
	long seq;
	distributedFDPars pars;
		
	if (dest_mask!=0)
	{
	
	pars.spec=spec;
	pars.deadline=deadline;
	strncpy(pars.modName,func.module,21);
	strncpy(pars.funcName,func.funcname,21);
	strncpy(pars.wcetName,func.wcetname,21);
	pars.start_ev=send_evt;
	pars.destination_port=dest_port;
	pars.to=dest_mask;    

	ret = call_tcb(start_distributedFD, descriptor, (distributedFDPars*)&pars, (long*)&seq, sizeof(distributedFDPars), sizeof(long));	

	if (ret) return ret;
	
	if (seq<0)
	 	return seq;	 	
	} 
	
	actionid->gstartInst = send_evt;

	ret = getTCBid(descriptor);
	
	if (ret<0) return ret;

	actionid->srcId = ret;
	actionid->seq = seq;
	actionid->destination_port = dest_port;

	return OK;
		 				
}

//Internal function
int notify_tcb_of_receipt(descriptor_t * descriptor,unsigned char from,unsigned long seq,unsigned long long ts)
{	
	msg_rcv_on_payloadPars pt;
	int res, ret;
	
	pt.seq=seq;
	pt.time=ts;
	pt.from=from;
		
	ret = call_tcb(msg_received_on_payload, descriptor, (msg_rcv_on_payloadPars*)&pt, (int*)&res, sizeof(distributedFDPars), sizeof(int));	

	if ( ret != 0)
		return ret;
	
	return res;  	
}

/*
==========================================================================

returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int endDistFD(descriptor_t * descriptor, distAction_id_t *actionid, TCBtimestamp * local_rcv_time, TCBtimestamp * global_rcv_time, TCBtimeinterval * procdelay){	
	int ret;

	ret = getTCBLocalAndGlobalTimestamp(descriptor,local_rcv_time,global_rcv_time);	
	
	if (ret<0) return ret;

//O local passara a ser a global !!!!
 	ret = notify_tcb_of_receipt(descriptor,actionid->srcId,actionid->seq,*local_rcv_time);
	if (ret<0) return ret;

	// no space inside the TCB - set as unknown
	if (ret == 1){
			*procdelay = 0xFFFFFFFFFFFFFFFF; 
	}
	else
		*procdelay=*global_rcv_time - actionid->gstartInst;

	return OK;	
}



/*
==========================================================================

returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/


int TCBwinfoport(unsigned int port, descriptor_t * descriptor) {
	int ret, dummy;		
															
	ret = call_tcb(winfo_to_fifo, descriptor, (unsigned int*)&port, (int*)&dummy, sizeof(unsigned int), sizeof(dummy));	

	if (ret != 0)
		return ret;
			
	return OK;
}


/*
==========================================================================
returns OK 
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
*/

int TCBwaitInfo(int wait_info_desc,fail_info_to_app* inf)
{
	if (read(wait_info_desc,inf,sizeof(fail_info_to_app)) <0 )  
		return TCB_ACESS_ERROR;
		
	return OK;		
}

/*
==========================================================================

returns OK
------
ERRORS
------
 TCB:
	   TCB_ACESS_ERROR	
 TTCB:
	  INVALID_PROTECTION
	  INVALID_ID	
*/

int TCBstopwinfoport(unsigned int port, descriptor_t * descriptor) {
	int ret_val, ret;
	fd_set fds;
	struct timeval tv;
	fail_info_to_app inf;
	int dummy;
	
	ret = call_tcb(stop_winfo_port, descriptor, (unsigned int*)&port, (int*)&dummy, sizeof(unsigned int), sizeof(int));	

	if (ret != 0)
		return ret;
			
	return OK;
}

#endif

/*
==========================================================================
*/

void printTCBerror(int err)
{
	printf("(TCB ERROR) > ");

	switch (err)
	{
	case	TCB_ACESS_ERROR:printf("Error on the access to TCB");break;
	case	NO_AVAIL_CONN  :printf("No available connection");break;
	case	NO_AVAIL_EXEC  :printf("No available timely execution"); break;
	case	CANT_RUN_TIMELY:printf("TCB Cannot run the function in a timely manner");break;		
	case	FUNC_NOT_FOUND :printf("Function not found");break;
	case	WCET_FUNC_NOT_FOUND :printf("Worst Case Execution Time function not found");break;
	case	NO_AVAIL_LOCAL_FD:printf("Service localFD not available");break;		
	case	ALREADY_FAILED:printf("Cannot timely detect the failure - already failed in the past");break;	
	case	INVALID_LOCALFD_DESC:printf("Invalid local failure detection descriptor");break;	
	case	NO_AVAIL_LOCAL_MEASURE:printf("No available local measurement"); break;
	case	INVALID_LOCALM_DESC:printf("Local measurement descriptor invalid"); break;
	case    TOO_MANY_DISTRIBUTED_FD:printf("Too many simultaneous requests to distributed failure detection service"); break;
        case    BURSTY_SENDS:printf("Bursty invocations to distributed failure detection service"); break;
        case    IN_INIT_STATE:printf("TCB is in initialization"); break;
	default:
#ifdef TTCB
	  print_error_TTCB(err);
#else
	  printf("Unknow error: %d\n",err);
#endif
	}
}

//=================================
// TTCB services
//=================================

#ifdef TTCB

#define HMAC_OPAD       0x5C
#define HMAC_IPAD       0x36

int LOG_LEVEL=1;
int DBG_LEVEL=1;

/*
+---------------------------------------+
|          TTCBEncrypted                |
|                                       |
| This function encrypts the data to    |
| send to the TTCB kernel, and also     |
| decrypts data received                |
|                                       |
| INPUT: input is pointer to clear text |
|        mode is 1 if pointer to char   |
| OUTPUT: ouptut is pointer to cyphered |
+---------------------------------------+
*/

/* 
+-------------------------------------------+
|   Define my own printing functions        |
|                                           |
|   These are similar to the ones defined   |
|  inside the kernel for code compatibility |
+-------------------------------------------+
*/

/* End printing functions */

void print_out(out_hdr_t *out)
{
        int j;
	printf("secret_mic ");
	for (j=0; j < sizeof(skey_t); j++) printf("%02X", out->secret_mic[j]);
	printf(" eid %d seq %d error %d\n", out->eid, out->seq, out->error);
} 


void call_TTCB(Tservice srvc, descriptor_t *descriptor, in_hdr_t *in, out_hdr_t *out, int size_in, int size_out)
{ 
        char snd[sizeof(ReqTcbHeader)+size_in];
	ReqTcbHeader msg;
	unsigned char mac[sizeof(skey_t)];
	MPI data;
	MPI final;
	int readsize;
	
	out->error = 0;
	msg.srvc=srvc;
	msg.sizeofmsg=size_in;

	if ((srvc != localAuthentication_enum) && (srvc != getLocalPublicKey_enum)) {
	  switch(descriptor->protection) {
	  case PROTECTION_NONE:
	//    printf("No protection for input.\n");
	    in->eid = descriptor->eid;
	    in->copy_eid = in->eid;
	    memcpy(in->secret_mic, descriptor->secret, sizeof(skey_t));
	    break;
	    
	  case PROTECTION_HMD5:
	    //printf("MD5 protection for input.\n");
	    in->eid = descriptor->eid;
	    in->copy_eid = in->eid;
	    in->seq = descriptor->seq;
	    descriptor->seq ++;
	    
	    hash_md5((unsigned char *)&(in->copy_eid), size_in - sizeof(eid_t)*2 - sizeof(skey_t), (unsigned char *)descriptor->secret, sizeof(skey_t), (unsigned char *)in->secret_mic);
	    break;
	    
	    
	  default: // should never happen since descriptor is setup by the api
	    out->error = INVALID_PROTECTION;
	    return;
	  }
	}
	
	memcpy(snd,&msg,sizeof(msg));
	memcpy((snd+sizeof(msg)), in, size_in);


	//printf("Going to write to TTCB[%i].\n",sizeof(snd));
	// call TTCB interface module
	if (write(descriptor->desc_orig, &snd, sizeof(snd)) < 0) 
		out->error = TCB_ACESS_ERROR;
	else
		if ((readsize=read(descriptor->desc_orig, out, size_out)) < 0) 
			out->error = TCB_ACESS_ERROR;
		else
		  {
		    //	    printf("Read from TTCB %i [%i]. Error is %d\n",readsize,size_out,out->error);
		  }


	if ((srvc != localAuthentication_enum) && (srvc != getLocalPublicKey_enum)) {
		switch(descriptor->protection) {
		          case PROTECTION_NONE:
			    //printf("No protection for output.\n");
			    if ((out->eid != descriptor->eid) || (memcmp(out->secret_mic, descriptor->secret, sizeof(skey_t)) != 0))
			      out->error = INVALID_ID;
			    break;
		          case PROTECTION_HMD5:
			  //printf("MD5 protection for output.\n");
				if ((out->eid != descriptor->eid) || (out->seq != descriptor->seq)) {
 					out->error = INVALID_ID;
					return;
				}

				hash_md5((unsigned char *)&(out->eid), size_out - sizeof(skey_t), (unsigned char *)descriptor->secret, sizeof(skey_t), mac); 

				if (memcmp(mac, out->secret_mic, sizeof(skey_t)) != 0)
					out->error = INVALID_ID;
				break;

			default:
				printf("Fatal error in call_TTCB\n"); // cannot happen
				exit(1);
		}
	}

	//printf("Sent and read from TTCB.\ncall_TTCB done.\n");
	return;
}





//-----------------
// Local Authentication service

int localAuthentication(descriptor_t *descriptor)
{
	LA_in_t in;
	LA_out_t out;
	int i;
	MPI toCipher;
	MPI Ciphered;
	int cipheredsize;
	MPI data;
	MPI result;
	ReqTcbHeader msg;
	char snd[200] = "\0";
	char *temppointer;
	int size_out;

	memset(snd,0,200);

	memcpy(in.key, descriptor->secret, sizeof(skey_t));
	in.protection = descriptor->protection;
	memcpy(in.challenge, descriptor->challenge, sizeof(challenge_t));


	if((in.protection != PROTECTION_NONE) && (in.protection != PROTECTION_HMD5)) {
	  return INVALID_PROTECTION;
	}

	/* Going to cipher in with TTCB's Public Key */
	
	toCipher=mpi_alloc(sizeof(in));
	Ciphered=mpi_alloc(64);
	
	toCipher->nlimbs=sizeof(in);
	memcpy(toCipher->d, &in, sizeof(in));
	
	public(Ciphered, toCipher, &(descriptor->TTCB_PublicKey));
	

	cipheredsize = (mpi_get_nbits(Ciphered)+7);
	cipheredsize = cipheredsize / 8; 
	
	msg.srvc=localAuthentication_enum;
	msg.sizeofmsg = 24 + cipheredsize;
	
	memcpy(snd,&msg,sizeof(msg));
	memcpy((snd+sizeof(msg)),Ciphered, 24);
	memcpy((snd + sizeof(msg)+ 24), Ciphered->d, cipheredsize);


	// call TTCB interface module
	if (write(descriptor->desc_orig, &snd, sizeof(msg)+ 24 + cipheredsize) < 0) 
	   return TCB_ACESS_ERROR;
	else
	  if ((size_out=read(descriptor->desc_orig, &out, 192)) < 0) 
	    return TCB_ACESS_ERROR;
	
	/* The out structure is encrypted with TTCB's secret key, so decrypt it to make sure it comes from TTCB */
	
	if (out.error < 0)
	  {
	    return out.error;
	  }
	
	data = mpi_alloc(64);
	result=mpi_alloc(32);
	
	temppointer=(char *)data->d;	    
	memcpy(data, out.challenge_sign, 24);
	data->d = (mpi_limb_t *) temppointer;
	memcpy(data->d, out.challenge_sign+24, 128);
	
	
	public(result, data, &(descriptor->TTCB_PublicKey));
	
	if(memcmp(descriptor->challenge,result->d,sizeof(challenge_t))!= 0) {  /* The unencrypted challenge is different than the one we proposed */
	  descriptor->eid = 0;
	  descriptor->seq = 0;
	  return FAILED_LOCAL_AUTHENTICATION;
	}
	
	descriptor->eid = out.eid;
	descriptor->seq = 1;
	
	return 0;
}



//-----------------
// Close connection with the TTCB

void closeTTCBConnection(descriptor_t *descriptor, error_out_t *out)
{
	eid_in_t in;

	if ((descriptor->desc_orig != 0) || (descriptor->wi_orig != 0))
		call_TTCB(end_connection_TTCB_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
        return;
}



//-----------------
// Agreement service
#ifdef DIST_AGREEMENT
void propose(descriptor_t *descriptor, elist_t elist, TCBtimestamp tstart, decision_t decision, value_t value, propose_out_t *out)
{
	propose_in_t in;

	memcpy(in.elist, elist, sizeof(elist_t));
	in.tstart = tstart;
	in.decision = decision;
	memcpy(in.value, value, sizeof(value_t));

	call_TTCB(propose_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
	return;
}



void decide(descriptor_t *descriptor, tag_t tag, decide_out_t *out)
{
	decide_in_t in;

	in.tag = tag;

	call_TTCB(decide_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
	return;
}

#endif //DIST_AGREEMENT


//-----------------
// Random Number service

#ifdef LOCAL_RANDOM

void getRandom(descriptor_t *descriptor, rand_out_t *out)
{
	eid_in_t in;

	call_TTCB(getRandom_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
	return;
}

#endif //LOCAL_RANDOM


//-----------------
// Other APIs

int getLocalPublicKey(descriptor_t * descriptor)
{
  eid_in_t in;
  char out[200];
  int sizeout;
  ReqTcbHeader msg;
  char error;
  char *temp_pointer;
  int tcbDesc, wiDesc;
  int i;
  int result;

  msg.srvc=getLocalPublicKey_enum;
  msg.sizeofmsg=0;

  // call TTCB interface module
  if (write(descriptor->desc_orig, &msg, sizeof(msg)) < 0) 
    return TCB_ACESS_ERROR;
  
  if ((sizeout=read(descriptor->desc_orig, &out, 200)) < 0) 
    return TCB_ACESS_ERROR;
  
  if(sizeout>128)
    {
      
      (descriptor->TTCB_PublicKey).n=mpi_alloc(128);
      temp_pointer=(char *)(descriptor->TTCB_PublicKey).n->d;
      memcpy((descriptor->TTCB_PublicKey).n, out, 24 );
      (descriptor->TTCB_PublicKey).n->d=(mpi_limb_t *)temp_pointer;
      memcpy((descriptor->TTCB_PublicKey).n->d, out+24, 128);

      (descriptor->TTCB_PublicKey).e=mpi_alloc(sizeout-128-48);      
      temp_pointer=(char *)(descriptor->TTCB_PublicKey).e->d;
      memcpy((descriptor->TTCB_PublicKey).e, out+128+24, 24 );
      (descriptor->TTCB_PublicKey).e->d=(mpi_limb_t *)temp_pointer;
      memcpy((descriptor->TTCB_PublicKey).e->d, out+128+48,sizeout-128-48 );
      result = 0;   

      
    } else 
      {
	result = -1;
      }

  return result;
}

void getLocalTTCBid(descriptor_t *descriptor, localTTCBid_out_t *out)
{
	eid_in_t in;

	//call_TTCB(end_connection_TTCB_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
	out->localTTCBid = (localTTCBid_t)getTCBid((descriptor_t *)descriptor->desc_orig);
	return;
}



void crashedLocalTTCBid(descriptor_t *descriptor, localTTCBid_t localTTCBid, error_out_t *out)
{
	localTTCBid_in_t in;

	in.localTTCBid = localTTCBid;

	call_TTCB(crashedLocalTTCBid_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
	return;
}



void crashedLocalTTCBeid(descriptor_t *descriptor,  eid_t eid, error_out_t *out)
{
	eid_in_t in;

	in.eid = eid;

	call_TTCB(crashedLocalTTCBeid_enum, descriptor, (in_hdr_t *)&in, (out_hdr_t *)out, sizeof(in), sizeof(*out));
	return;
}



void print_error_TTCB(int err)
{
        printf("(TTCB ERROR) > ");

        switch (err)
        {
        case    TCB_ACESS_ERROR: printf("Error on the access to TTCB");break;
        case    IN_INIT_STATE: printf("TTCB is initializing");break;
        case    NO_AVAIL_CONN: printf("No available connection");break;
        case    NO_AVAIL_LOCAL_MEASURE: printf("No available local measurement"); break;
        case    INVALID_LOCALM_DESC: printf("Local measurement descriptor invalid"); break;
        case    FUNC_NOT_FOUND : printf("Function not found");break;
        case    WCET_FUNC_NOT_FOUND : printf("Worst Case Execution Time function not found");break;
        case    NO_AVAIL_EXEC: printf("No available timely execution"); break;
        case    CANT_RUN_TIMELY: printf("The TTCB Cannot run the function in a timely manner");break;
        case    NO_AVAIL_LOCAL_FD: printf("Service localFD not available");break;
        case    ALREADY_FAILED: printf("Cannot timely detect the failure - already failed");break;
        case    INVALID_LOCALFD_DESC: printf("Invalid local failure detection descriptor");break;
        case    PARAM_OUT_OF_BOUND: printf("parameter out of bound");break;
        case    NO_ROOM_FOR_ENTITY: printf("no room for new entities in this local TTCB");break;
        case    EID_NOT_IN_ELIST: printf("eid not in elist");break;
        case    TSTART_EXPIRED: printf("tstart expired");break;
        case    MAX_NUM_AGREEMENTS_RUNNING: printf("maximum number of agreements already running");break;
        case    VALUE_ALREADY_PROPOSED: printf("trying to propose a value again for the same agreement");break;
        case    INVALID_TAG: printf("invalid tag (agreement result may have expired)");break;
        case    AGREEMENT_RUNNING: printf("agreement still running");break;
        case    INVALID_ID: printf("invalid eid, wrong secret, ...");break;
        case    INVALID_SEQ: printf("invalid sequence number");break;
        case    INVALID_DECISION: printf("invalid decision code");break;
	case	DID_NOT_PROPOSE: printf("the entity which value was choosen did not propose");break;
	case	NO_RESOURCES_PROPOSE: printf("no resources for a new agreement to run");break;
	case	AGREEMENT_EXPIRED: printf("the agreement results expired");break;
        case    INVALID_PROTECTION: printf("invalid protection parameter");break;
	case    FAILED_LOCAL_AUTHENTICATION: printf("Local Authentication failed");break;

        default: printf("unknown error code: %d", err);
        }

	printf("\n");
	return;
}

#endif
